#include "BrowserGui.h"
#include "HtmlCode.h"
#include <FastLED.h>
#include "LedControl.h"
#include <WiFiUDP.h>
#include "Storage.h"

BrowserGui& BrowserGui::getInstance() {
  static BrowserGui instance;
  return instance;
}

BrowserGui::BrowserGui() {
}

void BrowserGui::start() {
  Serial.println("Starting BrowserGui");
  localPort = 80;
  guiServer = new ESP8266WebServer(localPort);

  /*Menu*/
  guiServer->on("/", [&]() {
    showIndexPage();
  });

  guiServer->on("/displaysettings", [&]() {
    showDisplaySettingsPage();
  });

  guiServer->on("/wifidata", [&]() {
    showWifiDataPage();
  });

  guiServer->on("/settings", [&]() {
    showSettingsPage();
  });

  guiServer->on("/help", [&]() {
    showHelpPage();
  });

  guiServer->on("/reset", [&]() {
    guiServer->send(200, "text/html", "Die Uhr wird zurückgesetzt und startet gleich neu");
    intitializeDefaultValues();
  });

  /*actions*/
  guiServer->on("/setDisplaySettings", [&]()
  {
    onSetDisplaySettings();
  });

  guiServer->on("/setWifiData", [&]()
  {
    onSetWifiData();
  });

  guiServer->on("/setSettings", [&]()
  {
    onSetSettings();
  });

  /*Redirection is only necessary, when wifi is disconnected*/
  if (!wificontrol.isWifiConnected()) {
    guiServer->onNotFound([&]() {
      Serial.println("redirecting");
      homePage = htmlRedirect;
      homePage.replace("MAGICIPADDRESS", wificontrol.getCurrentIP().toString());
      guiServer->send(200, "text/html", homePage);
    });
  }

  guiServer->begin();
  Serial.print("BrowserGui started on port: ");
  Serial.println(localPort);

}

void BrowserGui::serverLoop() {
  guiServer->handleClient();
}

void BrowserGui::onSetDisplaySettings()
{
  String color_string = "";
  bool brightnessAutoHandled = false;

  if (guiServer->args() > 0 )
  {
    for ( uint8_t i = 0; i < guiServer->args(); i++ )
    {
      if (guiServer->argName(i) == "color")
      {
        color_string = guiServer->arg(i);
        ledcontrol.setColor(color_string);
      }

      if (guiServer->argName(i) == "brightness")
      {
        ambcontrol.setMinBrightness(guiServer->arg(i).toInt());
      }

      if (guiServer->argName(i) == "brightnessauto")
      {
        brightnessAutoHandled = true;
      }
    }
  }
  /*checkbox info is only sent, when checked*/
  if (brightnessAutoHandled)
  {
    ambcontrol.setBrightnessAuto(true);
  }
  else
  {
    ambcontrol.setBrightnessAuto(false);
  }
  showDisplaySettingsPage();
}

void BrowserGui::onSetWifiData() {
  String password;
  String ssid;

  Serial.println("WifiData action");

  if (guiServer->args() > 0)
  {
    for ( int i = 0; i < guiServer->args(); i++ )
    {
      if (guiServer->argName(i) == "psw")
      {
        password = guiServer->arg(i);
      }
      if (guiServer->argName(i) == "ssid")
      {
        ssid = guiServer->arg(i);
      }
    }
    wificontrol.setWifiData(ssid, password);
    guiServer->send(200, "text/html", "Die Uhr startet jetzt neu und verbindet sich mit ihrem WLAN.");
    delay(1000);
    ESP.reset();
  }
}

void BrowserGui::onSetSettings()
{
  String nightbegin = "23:00";
  String nightend = "06:00";
  bool summertimeAutoHandled = false;

  if (guiServer->args() > 0 )
  {
    for ( uint8_t i = 0; i < guiServer->args(); i++ )
    {
      if (guiServer->argName(i) == "nightbegin")
      {
        nightbegin = guiServer->arg(i);
      }
      if (guiServer->argName(i) == "nightend")
      {
        nightend = guiServer->arg(i);
      }
      if (guiServer->argName(i) == "summerauto")
      {
        summertimeAutoHandled = true;

      }
      if (guiServer->argName(i) == "timezone")
      {
        timecontrol.setTimeZone(guiServer->arg(i).toInt());
      }
      if (guiServer->argName(i) == "dreiviertelmode")
      {
        if (guiServer->arg(i) == "Dreiviertel")
        {
          ledcontrol.setDreiviertelMode(true);
        }
        else
        {
          ledcontrol.setDreiviertelMode(false);
        }
      }
    }
    ledcontrol.setNightmode(nightbegin, nightend);

    /*checkbox info is only sent, when checked*/
    if (summertimeAutoHandled)
    {
      timecontrol.setSummertimeAuto(true);
    }
    else
    {
      timecontrol.setSummertimeAuto(false);
    }
  }
  showSettingsPage();
}

void BrowserGui::showIndexPage()
{
  Serial.println("Index html");
  showDisplaySettingsPage();
}

void BrowserGui::showDisplaySettingsPage()
{
  /* Color String MUST be in format #RRGGBB*/
  String col = "#";
  uint8_t r = 0;
  uint8_t b = 0;
  uint8_t g = 0;

  loadColor(&r, &g, &b);
  /*if r == 0 this part of the string would be left out */
  if (0x10 > r)
  {
    col += "0";
  }
  col += String(r, HEX);
  if (0x10 > g)
  {
    col += "0";
  }
  col += String(g, HEX);
  if (0x10 > b)
  {
    col += "0";
  }
  col += String(b, HEX);

  Serial.println("DisplaySettings html");
  Serial.println("Colorstring: "+ col);
  homePage = htmlStartString + htmlDisplaySettingsString + htmlEndString;
  homePage.replace("MAGICCOLOR", col);
  if (ambcontrol.getBrightnessAuto())
  {
    homePage.replace("MAGICBRIGHTNESSAUTO", "checked");
  }
  else
  {
    homePage.replace("MAGICBRIGHTNESSAUTO", "");
  }
  homePage.replace("MAGICBRIGHTNESS", String(ambcontrol.getMinBrightness()));

  guiServer->send(200, "text/html", homePage);
}

void BrowserGui::showWifiDataPage()
{
  Serial.println("WifiData html");
  homePage = htmlStartString + htmlWifiString + htmlEndString;
  guiServer->send(200, "text/html", homePage);
}

void BrowserGui::showSettingsPage()
{
  /*time is stored in minutes  and has to be converted back to HH:MM format*/
  uint32_t nightbegin = 0;
  uint32_t nightend = 0;
  String beginString = "";
  String endString = "";

  loadNightBegin(&nightbegin);
  loadNightEnd(&nightend);

  if ( 10 > (nightbegin / 60))
  {
    beginString = "0";
  }
  beginString += String(nightbegin / 60);
  beginString += ":";
  if ( 10 > (nightbegin % 60))
  {
    beginString += "0";
  }
  beginString += String(nightbegin % 60);

  if ( 10 > (nightend / 60))
  {
    endString = "0";
  }
  endString += String(nightend / 60);
  endString += ":";
  if ( 10 > (nightend % 60))
  {
    endString += "0";
  }
    endString += String(nightend % 60);

  Serial.println("Settings html");
  homePage = htmlStartString + htmlSettingsString + htmlEndString;
  homePage.replace("MAGICNIGHTMODEBEGIN", beginString);
  homePage.replace("MAGICNIGHTMODEEND", endString);
  homePage.replace("MAGICUTC", String(timecontrol.getTimeZone()));

  if (timecontrol.getSummertimeAuto())
  {
    homePage.replace("MAGICSUMMERTIMEAUTO", "checked");
  }
  else
  {
    homePage.replace("MAGICSUMMERTIMEAUTO", "");
  }

  if (ledcontrol.getDreiviertelMode())
  {
    homePage.replace("MAGICDREIVIERTELMODE_DREIVIERTEL", "checked");
    homePage.replace("MAGICDREIVIERTELMODE_VIERTELVOR", "");
  }
  else
  {
    homePage.replace("MAGICDREIVIERTELMODE_VIERTELVOR", "checked");
    homePage.replace("MAGICDREIVIERTELMODE_DREIVIERTEL", "");
  }
  guiServer->send(200, "text/html", homePage);
}

void BrowserGui::showHelpPage()
{
  Serial.println("Help html");
  homePage = htmlStartString + htmlHelpString + htmlEndString;
  guiServer->send(200, "text/html", homePage);
}

