#include "TimeControl.h"
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include "Storage.h"
#define NTP_PACKET_SIZE 48

WiFiUDP Udp;
static const char ntpServerName[] = "de.pool.ntp.org";
unsigned int localPort = 8888;
int timeZone;
byte packetBuffer[NTP_PACKET_SIZE];

time_t getNtpTime();
void sendNTPpacket(IPAddress &address);

TimeControl& TimeControl::getInstance()
{
  static TimeControl instance;
  return instance;
}

TimeControl::TimeControl()
{
}

void TimeControl::start()
{
  loadUtcDiff(&timeZone);
  loadSummertimeAuto(&summerTimeAuto);
  Udp.begin(localPort);
  setSyncProvider(getNtpTime);
  setSyncInterval(3600);
}

void TimeControl::getTime(uint8_t *hh, uint8_t *mm, uint8_t *ss)
{
  *hh = hour();
  *mm = minute();
  *ss = second();
  if (summerTimeAuto && isSummertime())
  {
    *hh += 1;
    if(24 == *hh)
    {
      *hh = 0;
    }
  }
}

time_t getNtpTime()
{
  IPAddress ntpServerIP; // NTP server's ip address
  while (Udp.parsePacket() > 0) ; // discard any previously received packets
  // get a random server from the pool
  WiFi.hostByName(ntpServerName, ntpServerIP);
  Serial.print("ntpServerName: ");
  Serial.println(ntpServerName);
  Serial.print("ntpServerIP: ");
  Serial.println(ntpServerIP);
  sendNTPpacket(ntpServerIP);
  uint32_t beginWait = millis();
  while (millis() - beginWait < 5000) {
    int size = Udp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
      unsigned long secsSince1900;
      // convert four bytes starting at location 40 to a long integer
      secsSince1900 =  (unsigned long)packetBuffer[40] << 24;
      secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
      secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
      secsSince1900 |= (unsigned long)packetBuffer[43];
      return secsSince1900 - 2208988800UL + timeZone * SECS_PER_HOUR;
    }
  }
  Serial.println("ERROR NTP");
  return 0;
}


void sendNTPpacket(IPAddress &address)
{ // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}


bool TimeControl::isSummertime(void)
{
  if (month() < 3 || month() > 10) return false; // keine Sommerzeit in Jan, Feb, Nov, Dez
  if (month() > 3 && month() < 10) return true; // Sommerzeit in Apr, Mai, Jun, Jul, Aug, Sep
  if (month() == 3 && (hour() + 24 * day()) >= (1 + timeZone + 24 * (31 - (5 * year() / 4 + 4) % 7)) || month() == 10 && (hour() + 24 * day()) < (1 + timeZone + 24 * (31 - (5 * year() / 4 + 1) % 7)))
    return true;
  else
    return false;
}


void TimeControl::setTimeZone(int newTimezone)
{
  timeZone = newTimezone;
  saveUtcDiff(timeZone);
  setSyncProvider(getNtpTime);
}

int TimeControl::getTimeZone()
{
 return timeZone;
}


void TimeControl::setSummertimeAuto(bool newState)
{
 summerTimeAuto = newState;
 saveSummertimeAuto(summerTimeAuto);
}


bool TimeControl::getSummertimeAuto()
{
 return summerTimeAuto;
}
